/**
 * IE HTML5 elements fix - jQuery plugin
 * Copyright (c) 2010 Johan de Jong <http://johan.notitia.nl>
 *
 * Creative Commons Attribution-Share Alike 3.0
 * http://creativecommons.org/licenses/by-sa/3.0/
 *
 * How to use:
 *     - include the jQuery core as normal at the HEAD element
 *       <script type="text/javascript" src="/path/to/jquery.js"></script>
 *
 *     - include this file at the HEAD element
 *       <script type="text/javascript" src="/path/to/jquery.html5.js"></script>
 *
 *     - call the function at the HEAD element, *OUTSIDE* $(document).ready(); !!!
 *       <script type="text/javascript">
 *          $.html5();										// default usage
 *          $.html5('wrapper,content,copyright');			// with custom tags as a string, comma seperated
 *
 *          $(document).ready();							// now you can do the rest of your code
 *       </script>
 */
(function($){
	$.extend({
		html5 : function(tags){
			// only execute when used in Internet Explorer
			if($.browser.msie){
				// list of (known) HTML5 elements
				e = "abbr,article,aside,audio,canvas,datalist,details,eventsource,figure,footer,header,hgroup,mark,menu,meter,nav,output,progress,section,time,video";
				// if tags is set, add them to the elements list
				if(tags.length > 0) { e += ','+tags; }
				x = e;
				e = e.split(',');
				i = e.length;
				// loop through the elements and add them to the DOM
				// please note that no jQuery is used here, this due a bug with innerHTML
				while(i--){
					document.createElement(e[i]);
				}
				// create a new style element, insert the CSS rules and put it in the HEAD
				// please note that no jQuery is used here, this due a bug with innerHTML
				s = document.createElement('style');
				s.setAttribute("type", "text/css");
				s.styleSheet.cssText = x + ' {display:block;zoom:1;}';
				h = document.getElementsByTagName('head')[0];
				h.appendChild(s);
			}
		}
	})
})(jQuery);


/* 
	tidyHTML5MediaTags by Katja Deutschmann <http://www.katjadeutschmann.de/>
	
	This jQuery plugin recreates audio, video and source tags for older browsers like Firefox < 3.1 that otherwise nest the tags.
	
	@dependencies: jQuery <http://jquery.com>
	
	Function call: 
	$.tidyHTML5MediaTags();	
*/	
(function($){
	$.extend({
		tidyHTML5MediaTags: function() {
			var htmlParts = new Array();
			var sources = new Array();
			var lastElem;

			// only recreate tags if they're wrongly nested
			if ($('source source, video video, audio audio, video audio, audio video').length > 0) {
				recreateTags($('source').last());
			}

			function recreateTags(elem){
				var isAudio = elem.is('audio');
				var isVideo = elem.is('video');
				var isSource = elem.is('source');
				
				if (isAudio || isVideo) {
					// create new audio/video wit h the origin attributes and append source tags
					var medium = (isAudio) ? $('<audio/>') : $('<video/>');
					var attributes = elem[0].attributes;
					$.each(attributes, function(i, attribute) {
						medium.attr(attribute.name, attribute.value);
					});		
					$.each(sources, function(i, source) {
						medium.append(source);
					});
					// save audio/video
					htmlParts.unshift(medium);
					// empty sources for next audio/video
					sources = new Array();
					// run again if there are more source tags
					if (elem.siblings('source').length > 0) {
						lastElem = elem;
						recreateTags(elem.siblings('source'));		
					}
					// stop and replace the wrong old html
					else {
						$.each(htmlParts, function(i, part) {	
							elem.prev().after(part);
						});	
						elem.detach();
					}
				}
				else if (isSource){
					// create a new source tag with the origin attributes
					var newSource = $('<source/>');
					var attributes = elem[0].attributes;
					$.each(attributes, function(i, attribute) {
						newSource.attr(attribute.name, attribute.value);
					});
					// save source for parent audio/video
					sources.unshift(newSource);
					// get siblings of source tag except last audio/video (using parent().contents() to get also text nodes) and save them
					var siblings = elem.parent().contents().not(elem).not(lastElem);
					htmlParts.unshift(siblings);
					// recall with parent
					recreateTags(elem.parent());
				}
			}
		}
	})
})(jQuery);

(function($){
	$.html5('');
	$(document).ready(function(){
		$.tidyHTML5MediaTags();
	});
})(jQuery);