/** 
	HTML5AudioVideoFallback by Katja Brunner <https://katjabrunner.de/wissen/javascript/html5audiovideofallback.html>, 
	Default-Flash-Player for video fallback powered by Flowplayer (http://flowplayer.org/)
	
	@version: 						1.1
	@dependencies: 					jQuery <http://jquery.com>, 
									swfobject 2.2 <http://code.google.com/p/swfobject/> unless using option playerDirectEmbedCall, 
									enablehtml5.js (or something else enabling older browsers to interprete HTML5 media tags correctly)
	
	This jQuery plugin creates a fallback for HTML5 video or audios. As fallback the video/audio is played with a flash player (embedded by using SWFObject or
	by calling your provided embed call). If embedding is not possible download links will be provided.

	Function call: 
	$('myElement').HTML5AudioVideoFallback();

	@option idPrefix: 				A string prefix for the id of the flash container/object. If option generateId is true, a timestamp will be added.
	@option generateId:				If true the id for the flash container/object will be generated by adding a timestamp to the idPrefix. Default: true.
	@option flashPlayerSrc: 		The file path to the flash player (relative to the calling HTML file).
	@option flashExpressInstallSrc: The file path to the expressinstall.swf (relative to the calling HTML file).
	@option flashVersion: 			The flash version to use.
	@option initFunction: 			The name of the initializing function for the flash player (if there is one).
	@option width: 					The width of the flash player if the width is not provided as attribute of video or medium type is audio. Default: 600px.
	@option height: 				The height of the flash player if the height is not provided as attribute of video or medium type is audio. Default: 400px.
	@option flashMediumSrcType: 	The mimetype of the video/audio source that shall be used for the flash player (e.g. video/mp4, video/ogg, video/webm, 
									video/x-flv, audio/mpeg, audio/ogg). Default: video/mp4
									Please assure your flash player is able to play the file type.
	@option flashvars: 				The flashvars for the flash player. You can use these placeholders (see below): 
									###MEDIUM_SRC###, ###VIDEO_POSTER_SRC###, ###FLASH_ID###
	@option params: 				The params for the flash player. You can use these placeholders (see below): 
									###MEDIUM_SRC###, ###VIDEO_POSTER_SRC###, ###FLASH_ID###
	@option attributes: 			The attributes for the flash player. You can use these placeholders (see below): ###MEDIUM_SRC###, ###VIDEO_POSTER_SRC###, ###FLASH_ID###
	@option playerDirectEmbedCall:	A code expression that will be called in a function (e.g. "return $f(...)" for FlowPlayer. The return value will be set to data.flashMedium. 
									If playerDirectEmbedCall is not null it will be used for flash embedding instead of SWFObject.
									You can use these placeholders (see below):
									###FLASH_ID###, ###MEDIUM_SRC###, ###VIDEO_POSTER_SRC###, ###WIDTH###, ###HEIGHT###, ###FLASH_PLAYER_SRC###, ###FLASH_EXPRESS_INSTALL_SRC###, ###FLASH_VERSION###
	@option alternateMessageHTML: 	A HTML message that shall be shown in case flash cannot be embedded. You can use these placeholders (see below): 
									###ALTERNATE_MESSAGE_LINKS###, ###VIDEO_POSTER_SRC###
	@option alternateMessageLink: 	The HTML structure used for a link displayed in the alternateMessageHTML. You can use these placeholders (see below): 
									###MEDIUM_SRC###, ###MEDIUM_SRC_FILENAME###, ###MEDIUM_MIMETYPE###
	@option alwaysUseFlash:			Use flash instead of native HTML5 audio/video if true. Default: false.
	
	The following placeholders will be replaced by: 									
	###FLASH_PLAYER_SRC###: 		The file path to the flash player (relative to the calling HTML file).
	###FLASH_EXPRESS_INSTALL_SRC###:The file path to the expressinstall.swf (relative to the calling HTML file).
	###FLASH_VERSION###: 			The flash version to use.
	###MEDIUM_SRC###: 				The file path to the medium source.
	###MEDIUM_SRC_FILENAME###: 		The file name of the medium source.
	###MEDIUM_MIMETYPE###: 			The mimetype of the medium source.
	###VIDEO_POSTER_SRC###:			The file path to the poster source of a video.
	###FLASH_ID###: 				The id of the flash container/object.
	###WIDTH###: 					The width of the medium.
	###HEIGHT###: 					The height of the medium.
	###ALTERNATE_MESSAGE_LINKS###: 	A list of links to the video/audio sources for the alternate HTML message. Each link follows the pattern specified by alternateMessageLink.
	
	You can access all options via $('myElement').data('HTML5AudioVideoFallback').myDesiredOptionName. Furthermore you get access to:
	@data isFlashSupported: 		True if flash is supported, otherwise false.
	@data medium:					The medium jQuery object.
	@data mediumSources: 			The source jQuery objects or the medium jQuery object itself if medium has no source children.
	@data posterSrc:				The path to the video poster.
	@data flashId: 					The id of the flash. 
	@data flashMediumSrc:			The path to the file that shall played with flash.
	@data fallbackContainer:		The div container jQuery object containing the flash medium or download links message.
	@data flashMedium:				The flash medium object returned by SWFObject embedding or your direct embed call.
	
	The following methods can be called by $('myElement').HTML5AudioVideoFallback('myMethodName');
	@method remove:					Removes the alternate fallback content.

	Know issues:
	- Doesn't work in Safari Windows unless QuickTime is installed due to Safari insists on finding no source tags (native HTML5 does not work either in that case)
*/
(function($){
	var defaults = {
		idPrefix: 'HTML5AudioVideoFallback_',
		generateId: true,
		flashPlayerSrc: 'lib/flowplayer/flowplayer-3.2.7.swf',
		flashExpressInstallSrc: 'lib/expressinstall.swf',
		flashVersion: '9',
		initFunction: null,
		width: 600,
		height: 400,
		flashMediumSrcType: 'video/mp4',			
		flashvars: '',
		params: '{"flashvars": \"config={\'clip\': {\'url\': \'###MEDIUM_SRC###\', \'autoPlay\': 0}, \'canvas\': {\'backgroundColor\': \'#000000\', \'backgroundImage\': \'###VIDEO_POSTER_SRC###\'}}\"}',
		attributes: '',
		playerDirectEmbedCall: null,
		alternateMessageHTML: '<div><p>Unable to play medium. Please use download links provided:</p><ul class="linklist">###ALTERNATE_MESSAGE_LINKS###</ul></div>',
		alternateMessageLink: '<li><a href="###MEDIUM_SRC###">###MEDIUM_SRC_FILENAME###</a> (###MEDIUM_MIMETYPE###)</li>',
		alwaysUseFlash: false
	};
	
	// contains all methods that can be called by $.HTML5AudioVideoFallback('myMethodName');
	var methods = {		
		/**
		 * Calls init(context, options) to initialize fallback.
		 */ 
		init: function(options) {
			return this.each(function() {   
				init($(this), options);
			});
		},
		
		/**
		 * Calls remove(context) to remove fallback.
		 */
		remove: function() {
			return this.each(function() {   
				remove($(this));
			});
		}		
	};
		
	// constants
	var TYPE_VIDEO = 'video';
	var TYPE_AUDIO = 'audio';
	
	var MIMETYPE_THEORA = 'video/ogg';
	var MIMETYPE_H264 = 'video/mp4';
	var MIMETYPE_WEBMV = 'video/webm';
	var MIMETYPE_VORBIS = 'audio/ogg';
	var MIMETYPE_FLASH = 'video/x-flv';
	var MIMETYPE_M4A = 'audio/x-m4a';
	var MIMETYPE_MP3 = 'audio/mpeg';
	var MIMETYPE_WAV = 'audio/wav';
	var MIMETYPE_WEBMA = 'audio/webm';

    var fileExtensions = {
        video: {
            "ogg": MIMETYPE_THEORA,
            "ogv": MIMETYPE_THEORA,
            "avi": MIMETYPE_H264,
            "mp4": MIMETYPE_H264,
            "mkv": MIMETYPE_H264,
            "h264": MIMETYPE_H264,
            "264": MIMETYPE_H264,
            "avc": MIMETYPE_H264,
            "m4v": MIMETYPE_H264,
            "3gp": MIMETYPE_H264,
            "3gpp": MIMETYPE_H264,
            "3g2": MIMETYPE_H264,
            "mpg": MIMETYPE_H264,
            "mpeg": MIMETYPE_H264,
			"flv": MIMETYPE_FLASH,
			"webm": MIMETYPE_WEBMV
        },
        audio: {
            "ogg": MIMETYPE_VORBIS,
            "oga": MIMETYPE_VORBIS,
            "aac": MIMETYPE_M4A,
            "m4a": MIMETYPE_M4A,
            "mp3": MIMETYPE_MP3,
            "wav": MIMETYPE_WAV,
			"webm": MIMETYPE_WEBMA
        }
    };

	// check if flash plugin is available
	var flashSupported = false;
    if (navigator.plugins && navigator.plugins.length > 0) { 
        var type = 'application/x-shockwave-flash';
        var mimeTypes = navigator.mimeTypes;
        if (mimeTypes && mimeTypes[type] && mimeTypes[type].enabledPlugin && mimeTypes[type].enabledPlugin.description) {
        	flashSupported = true;
        }
    }
    else if (navigator.appVersion.indexOf('Mac') == -1 && window.execScript) {
        try {
        	new ActiveXObject('ShockwaveFlash.ShockwaveFlash');
        	flashSupported = true;
        } catch(error) {
        	flashSupported = false;
        }
    }	
    
	/**
	 * Initializes the fallback with the specified options.
	 * 
	 * @param context: The context.
	 * @param options: The options as object literal.
	 */
	function init(context, options) { 	
		// Merge options with default option values and make them accessable by data
		context.data('HTML5AudioVideoFallback', $.extend({}, defaults, options || {}));	
		var data = context.data('HTML5AudioVideoFallback');
		data.isFlashSupported = flashSupported;	

		// get video/audio tag
		data.medium = context;		
		var mediumNode = data.medium.get(0);
		// get type of medium (video or audio)
		var mediumType = mediumNode.nodeName.toLowerCase();
		
		// only use fallback if medium type is TYPE_AUDIO or TYPE_VIDEO
		if (mediumType == TYPE_AUDIO || mediumType == TYPE_VIDEO) {
			// get source elements or set medium itself as source if there are no source children
			data.mediumSources = data.medium.children('source');
			if (data.mediumSources.length == 0) {
				data.mediumSources = [data.medium];
			}

			// get flashMediumSrc, check if medium might be played by browser and if medium has type and src attributes specified
			data.flashMediumSrc = null;
			// canPlay will stay false if video/audio tag is not supported or none of the source types is possibly supported
			var canPlay = false;
			// isIncomplete will stay true if there aren't the attributes type and src
			var isIncomplete = true;
			for (var i = 0; i < data.mediumSources.length; i++) {
				var sourceSrc = $(data.mediumSources[i]).attr('src');
				var sourceType = $(data.mediumSources[i]).attr('type');
				// get mimetype out of src if type is undefined, but src is defined (e.g. if having no source children)
				if (typeof sourceType == 'undefined' && typeof sourceSrc != 'undefined') {
					sourceType = fileExtensions[mediumType][sourceSrc.split(".").slice(-1)[0]];
				}
				// check if there are the attributes type and src, set flashMediumSrc if type for flash matches
				if (typeof sourceType != 'undefined' && typeof sourceSrc != 'undefined') {
					if (sourceType.match(data.flashMediumSrcType, 'i')) {
						data.flashMediumSrc = sourceSrc;			
					}
					isIncomplete = false;
					// checks if video/audio tag is supported
					if (mediumNode.load && mediumNode.canPlayType) {
						// checks if the source type is probably supported
						if (mediumNode.canPlayType(sourceType) != '') {
							canPlay = true;
							break;
						}	
					}					
				}
			}
						
			// embed flash fallback if video/audio is not playable or alwaysUseFlash is true
			if ((!canPlay || data.alwaysUseFlash) && data.isFlashSupported && !isIncomplete && data.flashMediumSrc != null) {
				// get absolute poster src
				data.posterSrc = getAbsolutePath(data.medium.attr('poster'));
				// get absolute flash src
				data.flashMediumSrc = getAbsolutePath(data.flashMediumSrc);
	
				// get width and height
				data.width = (typeof data.medium.attr('width') == 'undefined') ? data.width : data.medium.attr('width');
				data.height = (typeof data.medium.attr('height') == 'undefined') ? data.height : data.medium.attr('height');				
	
				// create ID for flash container based on idPrefix and a timestamp
				data.flashId = (data.generateId) ? data.idPrefix + new Date().getTime() : data.idPrefix;		
				
				// create container for flash, hide medium and append flash container
				data.fallbackContainer = $('<div/>', {id: data.flashId});
				data.medium.hide();
				data.medium.before(data.fallbackContainer);
	
				// execute flash player init function if there is one
				if (data.initFunction != null && data.initFunction != '' && typeof data.initFunction == 'function') {
					data.initFunction.call();
				}
	
				// embed flash player with SWFObject if data.playerDirectEmbedCall is not specified
				if (data.playerDirectEmbedCall == null || data.playerDirectEmbedCall == '') {
					// replace placeholders of flashvars, params and attributes and parse them as JSON objects
					var flashvars = $.parseJSON(data.flashvars.replace(/###FLASH_ID###/g, data.flashId).replace(/###MEDIUM_SRC###/g, data.flashMediumSrc).replace(/###VIDEO_POSTER_SRC###/g, data.posterSrc));
					var params = $.parseJSON(data.params.replace(/###FLASH_ID###/g, data.flashId).replace(/###MEDIUM_SRC###/g, data.flashMediumSrc).replace(/###VIDEO_POSTER_SRC###/g, data.posterSrc)); 
					var attributes = $.parseJSON(data.attributes.replace(/###FLASH_ID###/g, data.flashId).replace(/###MEDIUM_SRC###/g, data.flashMediumSrc).replace(/###VIDEO_POSTER_SRC###/g, data.posterSrc));
	
					// callback function for SWFObject to provide download links if embedding flash fails (e.g. due to no flash plugin is installed)
					function showAlternateOnFailure(event) {				
						if (!event.success) {
							showAlternate(context);
						}
						else {
							// get flashMedium
							data.flashMedium = jQuery(event.ref);
						}
					}
	
					// embed flash player with SWFObject
					swfobject.embedSWF(data.flashPlayerSrc, data.flashId, data.width, data.height, data.flashVersion, data.flashExpressInstallSrc, flashvars, params, attributes, showAlternateOnFailure);					
				}
				// call data.playerDirectEmbedCall if it is specified
				else {
					var playerDirectEmbedCall = data.playerDirectEmbedCall.replace(/###FLASH_ID###/g, data.flashId).replace(/###MEDIUM_SRC###/g, data.flashMediumSrc).replace(/###VIDEO_POSTER_SRC###/g, data.posterSrc).replace(/###WIDTH###/g, data.width).replace(/###HEIGHT###/g, data.height).replace(/###FLASH_PLAYER_SRC###/g, data.flashPlayerSrc).replace(/###FLASH_EXPRESS_INSTALL_SRC###/g, data.flashExpressInstallSrc).replace(/###FLASH_VERSION###/g, data.flashVersion);
					try {
						data.flashMedium = new Function(playerDirectEmbedCall).call();
					}
					catch(error) {
						alert('HTML5AudioVideoFallback (playerDirectEmbedCall): \n' + error);
					}
				}
			}
			else if ((!canPlay && data.flashMediumSrc == null) || (!canPlay && !flashSupported)) {
				showAlternate(context);
			}		
		}
		else {
			data.fallbackContainer = null;
			data.flashMedium = null;
		}
	};
	
	/**
	 * Provides download links if embedding flash fails (e.g. due to no flash plugin is installed or no matching source for flash embedding).
	 * The message can be specified by options alternateMessageHTML and alternateMessageLink (see introducing comment).
	 * 
	 * @param context: The context.
	 */
	function showAlternate(context) {
		var data = getData(context);
		var alternateLinksList = '';
		$.each(data.mediumSources, function(i, mediumSource) {
			var mediumSourceSrc = $(mediumSource).attr('src');
			var mediumSourceType = $(mediumSource).attr('type');
			if (typeof mediumSourceSrc != 'undefined') {
				var alternateLink = data.alternateMessageLink.replace(/###MEDIUM_SRC###/g, mediumSourceSrc).replace(/###MEDIUM_SRC_FILENAME###/g, mediumSourceSrc.substring(mediumSourceSrc.lastIndexOf('/') + 1));
				if (typeof mediumSourceType != 'undefined') {
					alternateLink = alternateLink.replace(/###MEDIUM_MIMETYPE###/g, mediumSourceType);
				}
				else {
					alternateLink = alternateLink.replace(/###MEDIUM_MIMETYPE###/g, '');
				}
				alternateLinksList += alternateLink;
			}
		});
		
		var alternateHTML = data.alternateMessageHTML.replace(/###ALTERNATE_MESSAGE_LINKS###/g, alternateLinksList).replace(/###VIDEO_POSTER_SRC###/g, data.posterSrc);
		// set alternate message to fallbackContainer
		if (typeof data.fallbackContainer != 'undefined') {
			data.fallbackContainer.html(alternateHTML);
		}
		else {
			data.medium.hide();
			data.fallbackContainer = $(alternateHTML);
			data.medium.before(data.fallbackContainer);
		}
	}
	
	/**
	 * Gets the absolute path of a file.
	 * 
	 * @param path: The path to make absolute.
	 * @return The absolute path or null.
	 */
	function getAbsolutePath(path) {
		if (typeof path != 'undefined' && path != null) {
			// absolute path starting with / -> prepend domain
			if(path.charAt(0) == '/') {
				return document.location.protocol + '//' + document.location.host + path;
			}
			else {
				// absolute path starting with a protocol -> return path
				if(path.indexOf('://') > 0 && path.indexOf('://') < 10) {
					return path;
				}
				else {
					// relative path (base href specified) -> prepend base href
					if ($('base[href]').length > 0) {
						var basehref = $('base[href]').attr('href');
						if (basehref.charAt(basehref.length - 1) != '/') {
							basehref += '/';									
						}
						return basehref + path;
					}
					// relative path (no base href specified) -> prepend document.location.href
					else {
						return document.location.href.substring(0, document.location.href.lastIndexOf('/') + 1) + path;
					}
				}					
			}
		}
		else {
			return null;
		}
	}	
	
	/**
	 * Remove the fallback container with flash medium or message.
	 */ 
	function remove(context) {
		var data = getData(context);
		if (typeof data.fallbackContainer != 'undefined' && data.fallbackContainer != null) {
			data.fallbackContainer.remove();
		}
		$('#' + data.flashId).remove();
	}
	
	/**
	 * Gets data object out of context.
	 * 
	 * @param context: The context.
	 */
	function getData(context) {
		var data = context.data('HTML5AudioVideoFallback');
		if (!data) {
			init(context);
			data = context.data('HTML5AudioVideoFallback');
		}
		return data;
	}
	
	/**
	 * The jQuery method calling logic.
	 */
	$.fn.HTML5AudioVideoFallback = function(method) {  	
		// Method calling logic
		if (methods[method]) {
			return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
		} 
		else if (typeof method === 'object' || !method) {
			return methods.init.apply(this, arguments);
		} 
		else {
			$.error('Method ' +  method + ' does not exist on jQuery.HTML5AudioVideoFallback');
		}
	};
})(jQuery);
